"""Ariadne

一个优雅的 QQ Bot 框架.
"""
from typing import Literal, Optional, Type, overload

import graia.ariadne.event.lifecycle  # noqa: F401
import graia.ariadne.event.message  # noqa: F401
import graia.ariadne.event.mirai  # noqa: F401

from .app import Ariadne
from .typing import T


@overload
def get_running(type: Type[T] = Ariadne) -> T:
    ...


@overload
def get_running(type: Type[T], fail_err: Literal[False]) -> Optional[T]:
    ...


@overload
def get_running(type: Type[T], fail_err: Literal[True]) -> T:
    ...


def get_running(type: Type[T] = Ariadne, fail_err: bool = True) -> Optional[T]:
    """获取正在运行的实例

    Args:
        type (Type[T]): 实例类型
        fail_err (bool, optional): 如果没有正在运行的实例, 是否抛出异常

    Returns:
        T: 对应类型实例
    """
    from asyncio import AbstractEventLoop

    from graia.broadcast import Broadcast

    from .adapter import Adapter
    from .context import context_map

    if type in {Adapter, Ariadne, Broadcast, AbstractEventLoop}:
        if ctx := context_map.get(type.__name__):
            if val := ctx.get(None):
                return val
    for ariadne_inst in Ariadne.running:
        if type in ariadne_inst.info:
            return ariadne_inst.info[type]  # type: ignore
    if fail_err:
        raise ValueError(f"{type.__name__} is not running")
    return None



"""Ariadne 事件相关的导入集合"""

# no error

from graia.ariadne.event import MiraiEvent as MiraiEvent
from graia.ariadne.event.lifecycle import AdapterLaunched as AdapterLaunched
from graia.ariadne.event.lifecycle import AdapterShutdowned as AdapterShutdowned
from graia.ariadne.event.lifecycle import ApplicationLaunched as ApplicationLaunched
from graia.ariadne.event.lifecycle import (
    ApplicationLifecycleEvent as ApplicationLifecycleEvent,
)
from graia.ariadne.event.lifecycle import ApplicationShutdowned as ApplicationShutdowned
from graia.ariadne.event.message import FriendMessage as FriendMessage
from graia.ariadne.event.message import GroupMessage as GroupMessage
from graia.ariadne.event.message import OtherClientMessage as OtherClientMessage
from graia.ariadne.event.message import StrangerMessage as StrangerMessage
from graia.ariadne.event.message import TempMessage as TempMessage
from graia.ariadne.event.mirai import BotEvent as BotEvent
from graia.ariadne.event.mirai import (
    BotGroupPermissionChangeEvent as BotGroupPermissionChangeEvent,
)
from graia.ariadne.event.mirai import (
    BotInvitedJoinGroupRequestEvent as BotInvitedJoinGroupRequestEvent,
)
from graia.ariadne.event.mirai import BotJoinGroupEvent as BotJoinGroupEvent
from graia.ariadne.event.mirai import BotLeaveEventActive as BotLeaveEventActive
from graia.ariadne.event.mirai import BotLeaveEventKick as BotLeaveEventKick
from graia.ariadne.event.mirai import BotMuteEvent as BotMuteEvent
from graia.ariadne.event.mirai import BotOfflineEventActive as BotOfflineEventActive
from graia.ariadne.event.mirai import BotOfflineEventDropped as BotOfflineEventDropped
from graia.ariadne.event.mirai import BotOfflineEventForce as BotOfflineEventForce
from graia.ariadne.event.mirai import BotOnlineEvent as BotOnlineEvent
from graia.ariadne.event.mirai import BotReloginEvent as BotReloginEvent
from graia.ariadne.event.mirai import BotUnmuteEvent as BotUnmuteEvent
from graia.ariadne.event.mirai import CommandExecutedEvent as CommandExecutedEvent
from graia.ariadne.event.mirai import FriendEvent as FriendEvent
from graia.ariadne.event.mirai import (
    FriendInputStatusChangedEvent as FriendInputStatusChangedEvent,
)
from graia.ariadne.event.mirai import FriendNickChangedEvent as FriendNickChangedEvent
from graia.ariadne.event.mirai import FriendRecallEvent as FriendRecallEvent
from graia.ariadne.event.mirai import (
    GroupAllowAnonymousChatEvent as GroupAllowAnonymousChatEvent,
)
from graia.ariadne.event.mirai import (
    GroupAllowConfessTalkEvent as GroupAllowConfessTalkEvent,
)
from graia.ariadne.event.mirai import (
    GroupAllowMemberInviteEvent as GroupAllowMemberInviteEvent,
)
from graia.ariadne.event.mirai import (
    GroupEntranceAnnouncementChangeEvent as GroupEntranceAnnouncementChangeEvent,
)
from graia.ariadne.event.mirai import GroupEvent as GroupEvent
from graia.ariadne.event.mirai import GroupMuteAllEvent as GroupMuteAllEvent
from graia.ariadne.event.mirai import GroupNameChangeEvent as GroupNameChangeEvent
from graia.ariadne.event.mirai import GroupRecallEvent as GroupRecallEvent
from graia.ariadne.event.mirai import MemberCardChangeEvent as MemberCardChangeEvent
from graia.ariadne.event.mirai import MemberHonorChangeEvent as MemberHonorChangeEvent
from graia.ariadne.event.mirai import MemberJoinEvent as MemberJoinEvent
from graia.ariadne.event.mirai import MemberJoinRequestEvent as MemberJoinRequestEvent
from graia.ariadne.event.mirai import MemberLeaveEventKick as MemberLeaveEventKick
from graia.ariadne.event.mirai import MemberLeaveEventQuit as MemberLeaveEventQuit
from graia.ariadne.event.mirai import MemberMuteEvent as MemberMuteEvent
from graia.ariadne.event.mirai import MemberPerm as MemberPerm
from graia.ariadne.event.mirai import (
    MemberPermissionChangeEvent as MemberPermissionChangeEvent,
)
from graia.ariadne.event.mirai import (
    MemberSpecialTitleChangeEvent as MemberSpecialTitleChangeEvent,
)
from graia.ariadne.event.mirai import MemberUnmuteEvent as MemberUnmuteEvent
from graia.ariadne.event.mirai import NewFriendRequestEvent as NewFriendRequestEvent
from graia.ariadne.event.mirai import NudgeEvent as NudgeEvent
from graia.ariadne.event.mirai import OtherClientOfflineEvent as OtherClientOfflineEvent
from graia.ariadne.event.mirai import OtherClientOnlineEvent as OtherClientOnlineEvent
from graia.ariadne.event.mirai import RequestEvent as RequestEvent



"""Twilight: 混合式消息链处理器"""
import abc
import enum
import re
from argparse import Action
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    DefaultDict,
    Dict,
    Generic,
    Iterable,
    List,
    Optional,
    Type,
    TypedDict,
    TypeVar,
    Union,
    final,
    overload,
)

from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.exceptions import ExecutionStop
from graia.broadcast.interfaces.decorator import DecoratorInterface
from graia.broadcast.interfaces.dispatcher import DispatcherInterface
from pydantic.utils import Representation
from typing_extensions import Self

from ...typing import T, generic_isinstance, generic_issubclass
from ...util import gen_subclass
from ..chain import MessageChain
from ..element import Element
from .util import (
    CommandToken,
    ElementType,
    MessageChainType,
    TwilightParser,
    Unmatched,
    elem_mapping_ctx,
    split,
    tokenize_command,
    transform_regex,
)


class SpacePolicy(str, enum.Enum):
    """指示 RegexMatch 的尾随空格策略."""

    value: str

    NOSPACE = ""
    """禁止尾随空格"""

    PRESERVE = "( )?"
    """预留尾随空格"""

    FORCE = "( )"
    """强制尾随空格"""


NOSPACE = SpacePolicy.NOSPACE
PRESERVE = SpacePolicy.PRESERVE
FORCE = SpacePolicy.FORCE


# ANCHOR: Match


class Match(abc.ABC, Representation):
    """匹配项抽象基类"""

    dest: Union[int, str]

    def __init__(self) -> None:
        self._help = ""
        self.dest = ""

    def help(self, value: str) -> Self:
        """设置匹配项的帮助信息."""
        self._help = value
        return self

    def param(self, target: Union[int, str]) -> Self:
        """设置匹配项的分派位置."""
        self.dest = target
        return self

    def __matmul__(self, other: Union[int, str]) -> Self:
        return self.param(other)

    def __rmatmul__(self, other: Union[int, str]) -> Self:
        return self.param(other)

    def __rshift__(self, other: Union[int, str]) -> Self:
        return self.param(other)

    def __rlshift__(self, other: Union[int, str]) -> Self:
        return self.param(other)


T_Match = TypeVar("T_Match", bound=Match)


class MatchResult(Generic[T, T_Match], Representation):
    """匹配结果"""

    __slots__ = ("matched", "result", "origin")

    matched: bool
    """是否匹配成功"""

    result: Optional[T]
    """匹配结果"""

    origin: T_Match
    """原来的 Match 对象"""

    def __init__(self, matched: bool, origin: T_Match, result: T = None) -> None:
        """初始化 MatchResult 对象.

        Args:
            matched (bool): 是否匹配成功
            origin (T_Match): 原来的 Match 对象
            result (T, optional): 匹配结果. Defaults to None.
        """
        self.matched = matched
        self.origin = origin
        self.result = result


T_Result = TypeVar("T_Result", bound=MatchResult)


class RegexMatch(Match):
    """正则表达式匹配"""

    pattern: str
    """正则表达式字符串"""

    def __init__(self, pattern: str = "", optional: bool = False) -> None:
        """初始化 RegexMatch 对象.

        Args:
            pattern (str, optional): 正则表达式字符串. Defaults to "".
            optional (bool, optional): 是否可选. Defaults to False.
        Returns:
            None: 无返回.
        """
        super().__init__()
        self.pattern: str = pattern
        self._flags: re.RegexFlag = re.RegexFlag(0)
        self.optional: bool = optional
        self.space_policy: SpacePolicy = SpacePolicy.PRESERVE

    def flags(self, flags: re.RegexFlag) -> Self:
        """设置正则表达式的标志.

        Args:
            flags (re.RegexFlag): 正则表达式旗标.

        Returns:
            Self: RegexMatch 自身.
        """
        self._flags = flags
        return self

    def space(self, space: SpacePolicy) -> Self:
        """设置正则表达式的尾随空格策略.

        Args:
            space (SpacePolicy): 尾随空格策略.

        Returns:
            Self: RegexMatch 自身.
        """
        self.space_policy = space
        return self

    @final
    @property
    def _regex_str(self) -> str:
        """生成 RegexMatch 相应的正则表达式."""
        return (
            f"{transform_regex(self._flags, self._src)}"
            f"{'?' if self.optional else ''}{self.space_policy.value}"
        )

    @property
    def _src(self) -> str:
        """正则表达式的来源"""
        return self.pattern

    def __repr_args__(self):
        return [(None, self.pattern), ("space", self.space_policy.name), ("flags", self._flags)]


T_RegexMatch = TypeVar("T_RegexMatch", bound=RegexMatch)


class FullMatch(RegexMatch):
    """全匹配"""

    @property
    def _src(self) -> str:
        return re.escape(self.pattern)


class UnionMatch(RegexMatch):
    """多重匹配"""

    pattern: List[str]
    """匹配的选择项"""

    def __init__(
        self,
        *pattern: Union[str, Iterable[str]],
        optional: bool = False,
    ) -> None:
        """初始化 UnionMatch 对象.

        Args:
            *pattern (Union[str, Iterable[str]]): 匹配的选择项.
            optional (bool, optional): 匹配是否可选. Defaults to False.
        """
        super().__init__("", optional)
        self.pattern: List[str] = []
        for p in pattern:
            if isinstance(p, str):
                self.pattern.append(p)
            else:
                self.pattern.extend(p)
        self.optional = optional
        self.help(f"在 {self.pattern} 中选择一项")

    @property
    def _src(self) -> str:
        return f"{'|'.join(re.escape(i) for i in self.pattern)}"


class ElementMatch(RegexMatch):
    """元素类型匹配"""

    type: Type[Element]
    """要匹配的元素类型"""

    def __init__(
        self,
        type: Type[Element] = ...,
        optional: bool = False,
    ) -> None:
        """初始化 ElementMatch 对象.

        Args:
            type (Type[Element]): 元素类型.
            optional (bool, optional): 匹配是否可选. Defaults to False.
        """
        super(RegexMatch, self).__init__()
        self.type = type
        self.optional = optional
        self._flags: re.RegexFlag = re.RegexFlag(0)
        self.space_policy: SpacePolicy = SpacePolicy.PRESERVE
        self.help(f"{self.type.__name__} 元素")

    @property
    def _src(self) -> str:
        return f"\x02\\d+_{self.type.__fields__['type'].default}\x03"

    def __repr_args__(self):
        return [(None, self.type), ("space", self.space_policy.name), ("flags", self._flags)]


class ParamMatch(RegexMatch):
    """与 WildcardMatch 类似, 但需要至少一个字符. 且仅匹配用空格分开的一段"""

    def __init__(self, optional: bool = False) -> None:
        super().__init__(
            r"""(?:").+?(?:")|(?:').+?(?:')|[^ "']+""",
            optional,
        )
        self._help = "参数"

    def __repr_args__(self):
        return [(None, "PARAM"), ("space", self.space_policy.name), ("flags", self._flags)]


class WildcardMatch(RegexMatch):
    """泛匹配"""

    def __init__(self, greed: bool = True, optional: bool = False) -> None:
        """初始化 WildcardMatch 对象.

        Args:
            greed (bool, optional): 是否贪婪匹配. Defaults to True.
            optional (bool, optional): 匹配是否可选. Defaults to False.
        """
        super().__init__(f".*{'' if greed else'?'}", optional)


class ArgumentMatch(Match, Generic[T]):
    """参数匹配"""

    if TYPE_CHECKING:

        @overload
        def __init__(  # type: ignore
            self,
            *pattern: str,
            action: Union[str, Type[Action]] = ...,
            nargs: Union[int, str] = ...,
            const: T = ...,
            default: T = ...,
            type: Callable[[str], T] = ...,
            choices: Iterable[T] = ...,
            optional: bool = True,
        ):
            """初始化 ArgumentMatch 对象.

            Args:
                *pattern (str): 匹配的参数名.
                action (Union[str, Type[Action]], optional): 参数的动作. Defaults to "store".
                nargs (Union[int, str], optional): 参数的个数.
                const (T, optional): 参数的常量值.
                default (T, optional): 参数的默认值.
                type (Callable[[str], T], optional): 参数的类型.
                choices (Iterable[T], optional): 参数的可选值.
                optional (bool, optional): 参数是否可选. Defaults to True.
            Returns:
                None: 无返回
            """
            ...

    def __init__(self, *pattern: str, **kwargs) -> None:
        """初始化 ArgumentMatch 对象.

        Args:
            *pattern (str): 匹配的参数名.
            action (Union[str, Type[Action]], optional): 参数的动作. Defaults to "store".
            nargs (Union[int, str], optional): 参数的个数.
            const (T, optional): 参数的常量值.
            default (T, optional): 参数的默认值.
            type (Callable[[str], T], optional): 参数的类型.
            choices (Iterable[T], optional): 参数的可选值.
            optional (bool, optional): 参数是否可选. Defaults to True.
        Returns:
            None: 无返回
        """
        super().__init__()
        if not pattern:
            raise ValueError("pattern must not be empty")
        if not all(i.startswith("-") for i in pattern):
            raise ValueError("pattern must start with '-'")
        self.pattern: List[str] = list(pattern)
        self.arg_data: Dict[str, Any] = {"default": Unmatched}
        for k, v in kwargs.items():
            if k == "optional":
                self.arg_data["required"] = not v
            elif k == "type":
                if v is MessageChain:
                    v = MessageChainType()
                elif isinstance(v, Type) and issubclass(v, Element):
                    v = ElementType(v)
                self.arg_data["type"] = v
            else:
                self.arg_data[k] = v

        if "type" not in self.arg_data:
            self.arg_data["type"] = MessageChainType()

    def param(self, target: Union[int, str]) -> Self:
        self.arg_data["dest"] = target if isinstance(target, str) else f"_#!{target}!#_"
        return super().param(target)

    def help(self, value: str) -> Self:
        self.arg_data["help"] = value
        return super().help(value)

    def __repr_args__(self):
        return [(None, self.pattern)]


class ArgResult(Generic[T], MatchResult[T, ArgumentMatch]):
    """表示 ArgumentMatch 匹配结果"""

    ...


class RegexResult(MatchResult[MessageChain, RegexMatch]):
    """表示 RegexMatch 匹配结果"""

    ...


class ElementResult(MatchResult[Element, ElementMatch]):
    """表示 ElementMatch 匹配结果"""

    ...


class Sparkle(Representation):
    """Sparkle: Twilight 的匹配容器"""

    __slots__ = ("res",)

    def __init__(self, match_result: Dict[Union[int, str], MatchResult]):
        self.res = match_result

    @overload
    def __getitem__(self, item: Union[int, str]) -> MatchResult:
        ...

    @overload
    def __getitem__(self, item: Type[int]) -> List[MatchResult]:
        ...

    @overload
    def __getitem__(self, item: Type[str]) -> Dict[str, MatchResult]:
        ...

    def __getitem__(self, item: Union[int, str, Type[int], Type[str]]):
        if not isinstance(item, type):
            return self.get(item)
        if item is int:
            return [v for k, v in self.res.items() if isinstance(k, int)]
        elif item is str:
            return {k: v for k, v in self.res.items() if isinstance(k, str)}

    def get(self, item: Union[int, str]) -> MatchResult:
        return self.res[item]

    def __repr_args__(self):
        return [(repr(k), v) for k, v in self.res.items()]


T_Sparkle = TypeVar("T_Sparkle", bound=Sparkle)


class TwilightMatcher:
    """Twilight 匹配器"""

    def __init__(self, *root: Union[Iterable[Match], Match]):
        self._parser = TwilightParser(prog="", add_help=False)
        self._dest_map: Dict[str, ArgumentMatch] = {}
        self._group_map: Dict[int, RegexMatch] = {}
        self.dispatch_ref: Dict[Union[int, str], Match] = {}
        self.match_ref: DefaultDict[Type[Match], List[Match]] = DefaultDict(list)

        regex_str_list: List[str] = []
        regex_group_cnt: int = 0

        for i in root:
            if isinstance(i, Match):
                i = [i]
            for m in i:
                if isinstance(m, RegexMatch):
                    self.match_ref[RegexMatch].append(m)
                    if m.dest:
                        self._group_map[regex_group_cnt + 1] = m
                    regex_str_list.append(m._regex_str)
                    regex_group_cnt += re.compile(m._regex_str).groups

                elif isinstance(m, ArgumentMatch):
                    self.match_ref[ArgumentMatch].append(m)
                    if (
                        "action" in m.arg_data
                        and "type" in m.arg_data
                        and not self._parser.accept_type(m.arg_data["action"])
                    ):
                        del m.arg_data["type"]
                    action = self._parser.add_argument(*m.pattern, **m.arg_data)
                    if m.dest:
                        self._dest_map[action.dest] = m

                if m.dest:
                    if m.dest in self.dispatch_ref:
                        raise NameError(f"duplicate dispatch name: {m.dest}")
                    self.dispatch_ref[m.dest] = m

        self._regex_pattern: re.Pattern = re.compile("".join(regex_str_list))

    def match(
        self, arguments: List[str], elem_mapping: Dict[str, Element]
    ) -> Dict[Union[int, str], MatchResult]:
        """匹配参数
        Args:
            arguments (List[str]): 参数列表
            elem_mapping (Dict[str, Element]): 元素映射

        Returns:
            Dict[Union[int, str], MatchResult]: 匹配结果
        """
        result: Dict[Union[int, str], MatchResult] = {}
        if self._dest_map:
            namespace, arguments = self._parser.parse_known_args(arguments)
            nbsp_dict: Dict[str, Any] = namespace.__dict__
            for k, v in self._dest_map.items():
                res = nbsp_dict.get(k, Unmatched)
                result[v.dest] = MatchResult(res is not Unmatched, v, res)
        if not (total_match := self._regex_pattern.fullmatch(" ".join(arguments))):
            raise ValueError(f"{' '.join(arguments)} not matching {self._regex_pattern.pattern}")
        for index, match in self._group_map.items():
            group: Optional[str] = total_match.group(index)
            if group is None:
                res = None
            else:
                res = (
                    elem_mapping[group[1:-1].split("_")[0]]
                    if isinstance(match, ElementMatch)
                    else MessageChain._from_mapping_string(group, elem_mapping)
                )

            if match.dest:
                result[match.dest] = MatchResult(group is not None, match, res)
        return result

    def get_help(
        self,
        usage: str = "",
        description: str = "",
        epilog: str = "",
        dest: bool = True,
        sep: str = " -> ",
    ) -> str:
        """利用 Match 中的信息生成帮助字符串.

        Args:
            usage (str, optional): 使用方法 (命令格式).
            description (str, optional): 前导描述. Defaults to "".
            epilog (str, optional): 后置总结. Defaults to "".
            dest (bool, optional): 是否显示分派位置. Defaults to True.
            sep (str, optional): 分派位置分隔符. Defaults to " -> ".

        Returns:
            str: 生成的帮助字符串, 被格式化与缩进过了
        """

        formatter = self._parser._get_formatter()

        if usage:
            formatter.add_usage(None, self._parser._actions, [], prefix=usage + " ")

        formatter.add_text(description)

        _, optional, *_ = self._parser._action_groups

        if self.match_ref[RegexMatch]:
            formatter.start_section("匹配项")
            for match in self.match_ref[RegexMatch]:
                if match._help:
                    formatter.add_text(
                        f"""{ f"{match.dest}{sep}" if dest and match.dest else ""}{match._help}"""
                    )
            formatter.end_section()

        if self.match_ref[ArgumentMatch]:
            formatter.start_section("可选参数")
            formatter.add_arguments(optional._group_actions)
            formatter.end_section()

        formatter.add_text(epilog)

        # determine help from format above
        return formatter.format_help()

    def __repr__(self) -> str:
        return f"<Matcher {list(self._group_map.values()) + list(self._dest_map.values())!r}>"  # type: ignore

    def __str__(self) -> str:
        return repr(list(self._group_map.values()) + list(self._dest_map.values()))  # type: ignore


class _TwilightLocalStorage(TypedDict):
    result: Sparkle
    twilight: "Twilight"


class Twilight(Generic[T_Sparkle], BaseDispatcher):
    """暮光"""

    def __init__(
        self,
        *root: Union[Iterable[Match], Match],
        map_param: Optional[Dict[str, bool]] = None,
    ) -> None:
        """本魔法方法用于初始化本实例.

        Args:
            *root (Iterable[Match] | Match): 匹配规则.
            map_param (Dict[str, bool], optional): 向 MessageChain.asMappingString 传入的参数.
        """
        self.map_param = map_param or {}
        self.matcher: TwilightMatcher = TwilightMatcher(*root)

    def __repr__(self) -> str:
        return f"<Twilight: {self.matcher}>"

    def generate(self, chain: MessageChain) -> T_Sparkle:
        """从消息链生成 Sparkle 实例.

        Args:
            chain (MessageChain): 传入的消息链.

        Returns:
            T_Sparkle: 生成的 Sparkle 对象.
        """
        mapping_str, elem_mapping = chain._to_mapping_str(**self.map_param)
        token = elem_mapping_ctx.set(elem_mapping)
        arguments: List[str] = split(mapping_str, keep_quote=True)
        res = self.matcher.match(arguments, elem_mapping)
        elem_mapping_ctx.reset(token)
        return Sparkle(res)  # type: ignore

    @classmethod
    def from_command(  # ANCHOR: Sparkle: From command
        cls,
        command: str,
        extra_args: Optional[List[Match]] = None,
    ) -> "Twilight":
        """从 shell 式命令生成 Twilight.

        Args:
            command (str): 命令, 使用 {param} 或 {0} 的形式创建参数占位符. 使用 [a|b] 创建选择匹配. 使用 反斜杠 转义.

            extra_args (List[Match], optional): 可选的额外 Match 列表.

        Returns:
            Twilight: 生成的 Twilight.
        """
        extra_args = extra_args or []
        match: List[RegexMatch] = []

        for t_type, token_list in tokenize_command(command):
            if t_type is CommandToken.TEXT:
                match.append(FullMatch(*token_list).space(SpacePolicy.FORCE))
            elif t_type is CommandToken.CHOICE:
                match.append(UnionMatch(*token_list).space(SpacePolicy.FORCE))
            elif t_type is CommandToken.PARAM:
                match.append(ParamMatch().space(SpacePolicy.FORCE).param(token_list[0]))
            else:
                raise ValueError(f"unexpected token type: {t_type}")

        if match:
            match[-1].space_policy = SpacePolicy.NOSPACE

        return cls(*match, *extra_args)

    def get_help(
        self,
        usage: str = "",
        description: str = "",
        epilog: str = "",
        dest: bool = True,
        sep: str = " -> ",
    ) -> str:
        """利用 Match 中的信息生成帮助字符串.

        Args:
            usage (str, optional): 使用方法 (命令格式).
            description (str, optional): 前导描述. Defaults to "".
            epilog (str, optional): 后置总结. Defaults to "".
            dest (bool, optional): 是否显示分派位置. Defaults to True.
            sep (str, optional): 分派位置之间的分隔符. Defaults to " -> ".

        Returns:
            str: 生成的帮助字符串, 被格式化与缩进过了
        """
        return self.matcher.get_help(usage, description, epilog, dest, sep)

    async def beforeExecution(self, interface: DispatcherInterface):
        """检验 MessageChain 并将 Sparkle 存入本地存储

        Args:
            interface (DispatcherInterface): DispatcherInterface, 应该能从中提取 MessageChain

        Raises:
            ExecutionStop: 匹配以任意方式失败
        """
        local_storage: _TwilightLocalStorage = interface.local_storage  # type: ignore
        chain: MessageChain = await interface.lookup_param("message_chain", MessageChain, None)
        try:
            local_storage["result"] = self.generate(chain)
            local_storage["twilight"] = self
        except Exception as e:
            raise ExecutionStop from e

    async def catch(self, interface: DispatcherInterface):
        local_storage: _TwilightLocalStorage = interface.local_storage  # type: ignore
        sparkle = local_storage["result"]
        if generic_issubclass(Sparkle, interface.annotation):
            return sparkle
        if generic_issubclass(Twilight, interface.annotation):
            return self
        if interface.name in sparkle.res:
            result = sparkle.get(interface.name)
            if generic_isinstance(result.origin, interface.annotation):
                return result.origin
            if any(
                generic_issubclass(res_cls, interface.annotation) for res_cls in gen_subclass(MatchResult)
            ):
                return result
            if generic_isinstance(result.result, interface.annotation):
                return result.result


class ResultValue(Decorator):
    """返回 Match 结果值的装饰器"""

    pre = True

    @staticmethod
    async def target(i: DecoratorInterface):
        sparkle: Sparkle = i.local_storage["result"]
        res = sparkle.res.get(i.name, None)
        if generic_isinstance(res, i.annotation):
            return res
        raise ExecutionStop


class Help(Decorator):
    """返回帮助信息的装饰器"""

    pre = True

    if TYPE_CHECKING:

        @overload
        def __init__(  # type: ignore
            self,
            usage: str = "",
            description: str = "",
            epilog: str = "",
            dest: bool = True,
            sep: str = " -> ",
        ) -> None:
            ...

    def __init__(self, *args, **kwargs) -> None:
        self.args = args
        self.kwargs = kwargs

    async def target(self, i: DecoratorInterface):
        twilight: Twilight = i.local_storage["twilight"]
        # TODO: a better impl, support managing with other commands
        return twilight.get_help(*self.args, **self.kwargs)



"""Commander: 便捷的指令触发体系"""
import abc
import enum
import inspect
from contextlib import suppress
from contextvars import ContextVar
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Dict,
    Iterable,
    List,
    Optional,
    Sequence,
    Set,
    Type,
    TypeVar,
    Union,
    cast,
)

from graia.broadcast import Broadcast, Listener
from graia.broadcast.entities.decorator import Decorator
from graia.broadcast.entities.dispatcher import BaseDispatcher
from graia.broadcast.entities.exectarget import ExecTarget
from graia.broadcast.exceptions import ExecutionStop, RequirementCrashed
from pydantic import BaseModel, create_model, validator
from pydantic.fields import ModelField

from ...context import event_ctx
from ...dispatcher import ContextDispatcher
from ...event.message import MessageEvent
from ...model import AriadneBaseModel
from ...util import (
    ConstantDispatcher,
    const_call,
    eval_ctx,
    gen_subclass,
    resolve_dispatchers_mixin,
)
from ..chain import MessageChain
from ..element import Element
from ..parser.util import CommandToken, CommandTokenTuple, split, tokenize_command

T_Callable = TypeVar("T_Callable", bound=Callable)


def chain_validator(value: MessageChain, field: ModelField) -> Union[MessageChain, Element, str]:
    """
    MessageChain 处理函数.
    应用作 pydantic 的 Model validator.
    取决于字段类型标注, 若与消息链, 消息元素无关则会直接把消息链用 asDisplay 转换为字符串.

    Args:
        value (MessageChain): 消息链
        field (ModelField): 当前的 model 字段

    Returns:
        Union[MessageChain, Element, str]: 取决于字段类型标注
    """
    if field.outer_type_ is MessageChain:
        return value
    if issubclass(field.type_, Element):
        assert len(value) == 1
        assert isinstance(value[0], field.type_)
        return value[0]
    if isinstance(value, MessageChain):
        return value.asDisplay()
    if value is None:
        return field.default
    return value


class ParamDesc(abc.ABC):
    model: Optional[BaseModel]
    default: Any
    default_factory: Callable[[], Any]

    def __or__(self, other: "ParamDesc | Dict[str, Any]"):
        dct = other.__dict__ if isinstance(other, ParamDesc) else other
        for k, v in dct.items():
            if v and v is not ... and not isinstance(v, Decorator):
                self.__dict__[k] = v
        return self

    @abc.abstractmethod
    def gen_model(self, validators: Iterable[Callable]) -> None:
        """生成用于 pydantic 解析的 model 属性

        Args:
            validators (Iterable[Callable]): 用作 validator 的 Callable 可迭代对象
        """
        ...


@dataclass
class Slot(ParamDesc):
    """Slot"""

    def __init__(
        self,
        placeholder: Union[str, int],
        type: Type = ...,
        default: Any = ...,
        default_factory: Callable[[], Any] = ...,
    ) -> None:
        self.placeholder = placeholder
        self.type = type
        if self.type == "raw":
            self.type = _raw
        self.default = default
        self.default_factory = default_factory
        self.param_name: str = ""
        self.model: Optional[Type[BaseModel]] = None

    def gen_model(self, validators: Iterable[Callable]) -> None:
        if self.model or self.type is _raw:
            return

        self.default_factory = const_call(self.default) if self.default is not ... else self.default_factory

        if self.type is ...:
            self.type = MessageChain

        self.model = create_model(
            "SlotModel",
            __validators__={
                f"#validator_{i}#": validator("*", pre=True, allow_reuse=True)(v)
                for i, v in enumerate(validators)
            },
            val=(self.type, ...),  # default is handled at exec
        )


class Arg(ParamDesc):
    """Argument"""

    def __init__(
        self,
        pattern: str,
        type: Type[Union[BaseModel, Any]] = ...,
        default: Any = ...,
        default_factory: Callable[[], Any] = ...,
    ) -> None:

        self.pattern: str = pattern
        self.match_patterns: List[str] = []
        self.tags: List[str] = []
        self.default = default
        self.default_factory = default_factory
        self.param_name: Optional[str] = None

        tokens = tokenize_command(pattern)

        assert tokens[0][0] in {CommandToken.TEXT, CommandToken.CHOICE}, "Required argument pattern!"

        self.match_patterns = list(map(str, tokens[0][1]))

        for t_type, token_list in tokens[1:]:
            if t_type in {CommandToken.TEXT, CommandToken.CHOICE}:
                raise ValueError(
                    """Argument pattern can only be placed at head. """
                    """Use "{" and "}" for placeholders."""
                )
            if t_type is CommandToken.PARAM:
                if len(token_list) != 1:
                    raise ValueError("Arg doesn't support aliasing!")
                if str(token_list[0]) in self.tags:
                    raise ValueError("Duplicated tag!")
                self.tags.append(str(token_list[0]))

        self.nargs = len(self.tags)
        self.type = type
        self.model: Optional[Type[BaseModel]] = None

    def gen_model(self, validators: Iterable[Callable]) -> None:
        if self.model:
            return

        if self.nargs == 0:
            self.type = self.type if self.type is not ... else bool
            if self.default is ... and self.default_factory is ...:
                self.default = False
        elif self.nargs == 1:
            self.type = self.type if self.type is not ... else MessageChain
        self.default_factory = const_call(self.default) if self.default is not ... else self.default_factory

        if (
            isinstance(self.type, type)
            and issubclass(self.type, BaseModel)
            and not issubclass(self.type, AriadneBaseModel)
        ):
            self.model = self.type
            return

        if self.nargs == 0:  # Set default
            self.model = create_model(
                "ArgModel",
                __validators__={
                    f"#validator_{i}#": validator("*", pre=True, allow_reuse=True)(v)
                    for i, v in enumerate(validators)
                },
                val=(self.type, ...),
            )
        elif self.nargs == 1:
            self.model = create_model(
                "ArgModel",
                __validators__={
                    f"#validator_{i}#": validator("*", pre=True, allow_reuse=True)(v)
                    for i, v in enumerate(validators)
                },
                **{self.tags[0]: (self.type, ...)},
            )

        if self.model is ...:
            raise ValueError(f"You didn't supply a suitable model for {self.param_name}!")


@dataclass
class CommandPattern:
    """命令样式"""

    class ELast(enum.Enum):
        REQUIRED = "required"
        OPTIONAL = "optional"
        WILDCARD = "wildcard"

    token_list: "List[Set[str] | List[int | str]]"
    slot_map: Dict[Union[str, int], Slot]
    arg_map: Dict[str, Arg]
    last_type: ELast
    wildcard: str = ""


_raw = object()  # wildcard annotation object


class MismatchError(ValueError):
    """指令失配"""


commander_data_ctx: ContextVar[Dict[str, Any]] = ContextVar("commander_data_ctx", default={})


class CommandHandler(ExecTarget):
    """Command 的 ExecTarget 对象, 承担了参数提取等任务"""

    def __init__(
        self,
        record: CommandPattern,
        callable: Callable,
        dispatchers: Sequence[BaseDispatcher] = (),
        decorators: Sequence[Decorator] = (),
    ):
        super().__init__(
            callable,
            [
                ConstantDispatcher(commander_data_ctx),
                ContextDispatcher(),
                *resolve_dispatchers_mixin(dispatchers),
            ],
            list(decorators),
        )
        self.pattern: CommandPattern = record

    def get_data(
        self,
        slot_data: Dict[Union[int, str], MessageChain],
        arg_data: Dict[str, List[MessageChain]],
        wildcard_list: List[MessageChain],
    ) -> Dict[str, Any]:
        """基于 CommandRecord 与解析数据设置 ConstantDispatcher 参数

        Args:
            slot_data (Dict[Union[int, str], MessageChain]): Slot 的解析结果
            arg_data (Dict[str, List[MessageChain]]): Arg 的解析结果

        Returns:
            Dict[str, Any]: 参数
        """
        param_result: Dict[str, Any] = {}
        for arg in set(self.pattern.arg_map.values()):
            value = arg.default_factory()
            if TYPE_CHECKING:
                assert arg.param_name
                assert arg.model
            if arg.nargs:
                for param in arg.match_patterns:
                    if param in arg_data:
                        value = dict(zip(arg.tags, arg_data[param]))
                        break
                else:
                    if issubclass(arg.type, BaseModel) and isinstance(value, arg.type):
                        param_result[arg.param_name] = value
                        continue
                    if not isinstance(value, list):
                        value = [value]
                    if not isinstance(value, dict):
                        value = dict(zip(arg.tags, value))

                if not issubclass(arg.type, BaseModel) or issubclass(arg.type, AriadneBaseModel):
                    param_result[arg.param_name] = arg.model(**value).__dict__[arg.tags[0]]
                else:
                    param_result[arg.param_name] = arg.model(**value)

            else:
                if any(param in arg_data for param in arg.match_patterns):
                    value = not value
                param_result[arg.param_name] = arg.model(val=value).__dict__["val"]

        for ind, slot in self.pattern.slot_map.items():
            if TYPE_CHECKING:
                assert slot.model
            if slot.param_name != self.pattern.wildcard:
                value = slot_data.get(ind, None) or slot.default_factory()
                param_result[slot.param_name] = slot.model(val=value).__dict__["val"]
            else:
                if slot.type is _raw:
                    param_result[slot.param_name] = MessageChain([" "]).join(wildcard_list, merge=True)
                else:
                    param_result[slot.param_name] = tuple(
                        slot.model(val=chain).__dict__["val"] for chain in wildcard_list
                    )
        return param_result


class Commander:
    """便利的指令触发体系"""

    def __init__(self, broadcast: Broadcast, listen: bool = True):
        """
        Args:
            broadcast (Broadcast): 事件系统
            listen (bool): 是否监听指令
        """
        self.broadcast = broadcast
        self.command_handlers: List[CommandHandler] = []
        self.validators: List[Callable] = [chain_validator]

        async def execute_func(chain: MessageChain):
            await self.execute(chain)

        self.listen_func = execute_func

        if listen:
            self.broadcast.listeners.append(
                Listener(
                    self.listen_func,
                    self.broadcast.getDefaultNamespace(),
                    list(gen_subclass(MessageEvent)),
                )
            )

    def __del__(self):
        self.broadcast.listeners = [i for i in self.broadcast.listeners if i.callable != self.listen_func]

    def add_type_cast(self, *caster: Callable):
        """添加类型验证器 (type caster / validator)"""
        self.validators = [*reversed(caster), *self.validators]

    def command(
        self,
        command: str,
        setting: Optional[Dict[str, Union[Slot, Arg]]] = None,
        dispatchers: Sequence[BaseDispatcher] = (),
        decorators: Sequence[Decorator] = (),
    ) -> Callable[[T_Callable], T_Callable]:
        """装饰一个命令处理函数

        Args:
            command (str): 要处理的命令
            setting (Dict[str, Union[Slot, Arg]], optional): 参数设置.
            dispatchers (Sequence[BaseDispatcher], optional): 可选的额外 Dispatcher 序列.
            decorators (Sequence[Decorator], optional): 可选的额外 Decorator 序列.

        Raises:
            ValueError: 命令格式错误

        Returns:
            Callable[[T_Callable], T_Callable]: 装饰器
        """

        slot_map: Dict[Union[int, str], Slot] = {}
        pattern_arg_map: Dict[str, Arg] = {}
        param_arg_map: Dict[str, Arg] = {}
        for name, val in (setting or {}).items():
            if isinstance(val, Slot):
                slot_map[val.placeholder] = val
            elif isinstance(val, Arg):
                for pattern in val.match_patterns:
                    pattern_arg_map[pattern] = val
                param_arg_map[name] = val
            else:
                raise TypeError(f"Unknown setting value: {name} - {val!r}")
            val.param_name = name

        token_list: "List[Set[str] | List[int | str]]" = []  # set: const, list: param

        placeholder_set: Set[Union[int, str]] = set()

        last: CommandPattern.ELast = CommandPattern.ELast.REQUIRED
        wildcard_slot_name: str = ""

        command_tokens: List[CommandTokenTuple] = tokenize_command(command)

        for (t_type, tokens) in command_tokens:
            if t_type in {CommandToken.TEXT, CommandToken.CHOICE}:
                assert not any(
                    token in pattern_arg_map for token in tokens
                ), f"{tokens} conflicts with a Arg object!"

                token_list.append(set(cast(List[str], tokens)))

            elif t_type is CommandToken.ANNOTATED:
                wildcard, name, annotation, default = cast(List[str], tokens)
                if wildcard or default:
                    assert (
                        tokens is command_tokens[-1][1]
                    ), "Not setting wildcard / optional on the last slot!"
                if wildcard:
                    last = CommandPattern.ELast.WILDCARD
                if default:
                    last = CommandPattern.ELast.OPTIONAL
                assert name not in placeholder_set, "Duplicated parameter slot!"
                placeholder_set.add(name)
                parsed_slot = Slot(
                    name,
                    eval(annotation or "...", *eval_ctx(1, {"raw": _raw})),
                    eval(default or "...", *eval_ctx(1)),
                )
                parsed_slot.param_name = name  # assuming that param_name is consistent
                if name in slot_map:
                    slot_map[name] = parsed_slot | slot_map[name]  # parsed slot < provided slot
                if wildcard:
                    wildcard_slot_name = name
                token_list.append([name])
            elif t_type is CommandToken.PARAM:
                for param_name in tokens:
                    assert param_name not in placeholder_set, "Duplicated parameter slot!"
                    placeholder_set.add(param_name)
                token_list.append(tokens)

        def wrapper(func: T_Callable) -> T_Callable:
            """register as command executor"""

            # scan function signature

            def __translate_obj(obj):
                if obj is inspect.Parameter.empty:
                    return ...
                if isinstance(obj, Decorator):
                    return ...
                return obj

            for name, parameter in inspect.signature(func).parameters.items():
                annotation, default = __translate_obj(parameter.annotation), __translate_obj(
                    parameter.default
                )
                if name in placeholder_set:
                    parsed_slot = Slot(name, annotation, default)
                    parsed_slot.param_name = name  # assuming that param_name is consistent
                    if name in slot_map:
                        slot_map[name] = parsed_slot | slot_map[name]  # parsed slot < provided slot
                    if default is not ...:
                        assert all(
                            [
                                slot_map[name].placeholder in command_tokens[-1][1],
                                command_tokens[-1][0]
                                in {
                                    CommandToken.ANNOTATED,
                                    CommandToken.PARAM,
                                },
                            ]
                        ), "Not setting wildcard / optional on the last slot!"

                        nonlocal last
                        if last is CommandPattern.ELast.REQUIRED:
                            last = CommandPattern.ELast.OPTIONAL

                if name in param_arg_map:
                    arg = param_arg_map[name]
                    arg.type = arg.type if arg.type is not ... else parameter.annotation
                    if arg.default is ... and arg.default_factory is ...:
                        arg.default = parameter.default

            for slot in slot_map.values():
                slot.gen_model(self.validators)
            for arg in param_arg_map.values():
                arg.gen_model(self.validators)

            self.command_handlers.append(
                CommandHandler(
                    CommandPattern(token_list, slot_map, pattern_arg_map, last, wildcard_slot_name),
                    func,
                    dispatchers,
                    decorators,
                )
            )

            return func

        return wrapper

    async def execute(self, chain: MessageChain):
        """触发 Commander.

        Args:
            chain (MessageChain): 触发的消息链
        """

        mapping_str, elem_m = chain._to_mapping_str()

        for handler in reversed(self.command_handlers):  # starting from latest added
            pattern = handler.pattern
            text_index: int = 0
            token_index: int = 0
            arg_data: Dict[str, List[MessageChain]] = {}
            slot_data: Dict[Union[str, int], MessageChain] = {}
            text_list: List[str] = split(mapping_str)
            wildcard_list: List[MessageChain] = []
            with suppress(IndexError, MismatchError, ValueError, RequirementCrashed, ExecutionStop):
                while text_index < len(text_list):
                    text = text_list[text_index]
                    text_index += 1

                    if text in pattern.arg_map:  # Arg handle
                        if text in arg_data:
                            raise MismatchError("Duplicated argument")
                        nargs: int = pattern.arg_map[text].nargs
                        arg_data[text] = [
                            MessageChain._from_mapping_string(t, elem_m)
                            for t in text_list[text_index : text_index + nargs]
                        ]
                        text_index += nargs

                    else:  # Constant and Slot handle
                        tokens = pattern.token_list[token_index]
                        token_index += 1
                        if isinstance(tokens, set) and text not in tokens:
                            raise MismatchError
                        if isinstance(tokens, list):
                            if pattern.last_type is CommandPattern.ELast.WILDCARD and token_index == len(
                                pattern.token_list
                            ):
                                wildcard_list.append(MessageChain._from_mapping_string(text, elem_m))
                                token_index -= 1
                            for slot in tokens:
                                slot_data[slot] = MessageChain._from_mapping_string(text, elem_m)

                if text_index < len(pattern.token_list) - (
                    pattern.last_type is not CommandPattern.ELast.REQUIRED
                ):
                    continue

                dispatchers = []
                if event := event_ctx.get(None):
                    dispatchers = resolve_dispatchers_mixin([event.Dispatcher])
                token = commander_data_ctx.set(handler.get_data(slot_data, arg_data, wildcard_list))
                await self.broadcast.Executor(handler, dispatchers)
                commander_data_ctx.reset(token)



"""Ariadne 各种 model 存放的位置"""
import functools
import json
from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import TYPE_CHECKING, Dict, Optional, Union

from graia.broadcast.entities.listener import Listener
from loguru import logger
from pydantic import BaseConfig, BaseModel, Extra, Field, validator
from pydantic.networks import AnyHttpUrl
from typing_extensions import Literal
from yarl import URL

from graia.ariadne.util import gen_subclass

if TYPE_CHECKING:
    from .app import Ariadne
    from .message.chain import MessageChain
    from .typing import AbstractSetIntStr, DictStrAny, MappingIntStrAny


def datetime_encoder(v: datetime) -> float:
    """编码 datetime 对象

    Args:
        v (datetime): datetime 对象

    Returns:
        float: 编码后的 datetime (时间戳)
    """
    return v.timestamp()


class DatetimeEncoder(json.JSONEncoder):
    """可以编码 datetime 的 JSONEncoder"""

    def default(self, o):
        return int(o.timestamp()) if isinstance(o, datetime) else super().default(o)


class AriadneBaseModel(BaseModel):
    """
    Ariadne 一切数据模型的基类.
    """

    def dict(  # type: ignore
        self,
        *,
        include: Union[None, "AbstractSetIntStr", "MappingIntStrAny"] = None,
        exclude: Union[None, "AbstractSetIntStr", "MappingIntStrAny"] = None,
        by_alias: bool = False,
        skip_defaults: bool = False,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> "DictStrAny":
        _, *_ = by_alias, exclude_none, skip_defaults
        return super().dict(
            include=include,  # type: ignore
            exclude=exclude,  # type: ignore
            by_alias=True,
            skip_defaults=False,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=True,
        )

    class Config(BaseConfig):
        """Ariadne BaseModel 设置"""

        extra = Extra.allow
        json_encoders = {
            datetime: datetime_encoder,
        }
        arbitrary_types_allowed = True


@dataclass
class ChatLogConfig:
    """配置日志如何记录 QQ 消息与事件."""

    enabled: bool = True
    """是否开启聊天日志"""

    log_level: str = "INFO"
    """聊天日志的 log 等级"""

    group_message_log_format: str = (
        "{bot_id}: [{group_name}({group_id})] {member_name}({member_id}) -> {message_string}"
    )
    """群消息格式"""

    friend_message_log_format: str = "{bot_id}: [{friend_name}({friend_id})] -> {message_string}"
    """好友消息格式"""

    temp_message_log_format: str = (
        "{bot_id}: [{group_name}({group_id}).{member_name}({member_id})] -> {message_string}"
    )
    """临时消息格式"""

    other_client_message_log_format: str = "{bot_id}: [{platform_name}({platform_id})] -> {message_string}"
    """其他客户端消息格式"""

    stranger_message_log_format: str = "{bot_id}: [{stranger_name}({stranger_id})] -> {message_string}"
    """陌生人消息格式"""

    active_message_log_format: str = "{bot_id}: {sync_label}[{subject}] <- {message_string}"

    def initialize(self, app: "Ariadne"):
        """利用 Ariadne 对象注册事件日志处理器"""
        from .event.message import (
            ActiveMessage,
            FriendMessage,
            GroupMessage,
            OtherClientMessage,
            StrangerMessage,
            TempMessage,
        )

        @app.broadcast.receiver(GroupMessage, -1)
        def log_group_message(event: GroupMessage):
            logger.log(
                self.log_level,
                self.group_message_log_format.format(
                    group_id=event.sender.group.id,
                    group_name=event.sender.group.name,
                    member_id=event.sender.id,
                    member_name=event.sender.name,
                    member_permission=event.sender.permission.name,
                    bot_id=app.mirai_session.account,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        @app.broadcast.receiver(FriendMessage, -1)
        def log_friend_message(event: FriendMessage):
            logger.log(
                self.log_level,
                self.friend_message_log_format.format(
                    bot_id=app.mirai_session.account,
                    friend_name=event.sender.nickname,
                    friend_id=event.sender.id,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        @app.broadcast.receiver(TempMessage, -1)
        def log_temp_message(event: TempMessage):
            logger.log(
                self.log_level,
                self.temp_message_log_format.format(
                    group_id=event.sender.group.id,
                    group_name=event.sender.group.name,
                    member_id=event.sender.id,
                    member_name=event.sender.name,
                    member_permission=event.sender.permission.name,
                    bot_id=app.mirai_session.account,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        @app.broadcast.receiver(StrangerMessage, -1)
        def log_stranger_message(event: StrangerMessage):
            logger.log(
                self.log_level,
                self.stranger_message_log_format.format(
                    bot_id=app.mirai_session.account,
                    stranger_name=event.sender.nickname,
                    stranger_id=event.sender.id,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        @app.broadcast.receiver(OtherClientMessage, -1)
        def log_other_client_message(event: OtherClientMessage):
            logger.log(
                self.log_level,
                self.other_client_message_log_format.format(
                    bot_id=app.mirai_session.account,
                    platform_name=event.sender.platform,
                    platform_id=event.sender.id,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        def log_active_message(event: ActiveMessage):
            logger.log(
                self.log_level,
                self.active_message_log_format.format(
                    bot_id=app.mirai_session.account,
                    sync_label="[SYNC]" if event.sync else "",
                    subject=event.subject,
                    message_string=event.messageChain.asDisplay().__repr__(),
                ),
            )

        app.broadcast.listeners.append(
            Listener(
                log_active_message,
                app.broadcast.getDefaultNamespace(),
                list(gen_subclass(ActiveMessage)),
                priority=-1,
            )
        )


class MiraiSession(AriadneBaseModel):
    """
    用于描述与上游接口会话, 并存储会话状态的实体类.

    Attributes:
        host (AnyHttpUrl): `mirai-api-http` 服务所在的根接口地址
        account (int): 应用所使用账号的整数 ID, 虽然启用 `singleMode` 时不需要, 但仍然建议填写.
        verify_key (str): 在 `mirai-api-http` 配置流程中定义, 需为相同的值以通过安全验证, 需在 mirai-api-http 配置里启用 `enableVerify`.
        session_key (str, optional): 会话标识, 即会话中用于进行操作的唯一认证凭证.
    """

    host: Optional[AnyHttpUrl]
    """链接地址, 以 http 开头, 作为服务器连接时应为 None"""

    single_mode: bool = False
    """mirai-console 是否开启 single_mode (单例模式)"""

    account: Optional[int] = None
    """账号"""

    verify_key: Optional[str] = None
    """mirai-api-http 配置的 VerifyKey 字段"""

    session_key: Optional[str] = None
    """会话标识"""

    version: Optional[str] = None
    """mirai-api-http 的版本"""

    def __init__(
        self,
        host: Optional[Union[AnyHttpUrl, str]] = None,
        account: Optional[Union[int, str]] = None,
        verify_key: Optional[str] = None,
        *,
        single_mode: bool = False,
    ) -> None:
        super().__init__(
            host=host,  # type: ignore
            account=account,  # type: ignore
            verify_key=verify_key,  # type: ignore
            single_mode=single_mode,  # type: ignore
        )

    def url_gen(self, route: str) -> str:
        """生成 route 对应的 API URI

        Args:
            route (str): route 地址

        Returns:
            str: 对应的 API URI
        """
        if self.host is None:
            raise ValueError("Remote host is unset")
        return str(URL(self.host) / route)


@functools.total_ordering
class MemberPerm(Enum):
    """描述群成员在群组中所具备的权限"""

    Member = "MEMBER"  # 普通成员
    Administrator = "ADMINISTRATOR"  # 管理员
    Owner = "OWNER"  # 群主

    def __str__(self) -> str:
        return self.value

    def __lt__(self, other: "MemberPerm"):
        lv_map = {MemberPerm.Member: 1, MemberPerm.Administrator: 2, MemberPerm.Owner: 3}
        return lv_map[self] < lv_map[other]

    def __repr__(self) -> str:
        perm_map: Dict[str, str] = {
            "MEMBER": "<普通成员>",
            "ADMINISTRATOR": "<管理员>",
            "OWNER": "<群主>",
        }
        return perm_map[self.value]


class Group(AriadneBaseModel):
    """描述 Tencent QQ 中的群组."""

    id: int
    """群号"""

    name: str
    """群名"""

    accountPerm: MemberPerm = Field(..., alias="permission")
    """你在群中的权限"""

    def __int__(self):
        return self.id

    def __str__(self) -> str:
        return f"{self.name}({self.id})"

    async def getConfig(self) -> "GroupConfig":
        """获取该群组的 Config

        Returns:
            Config: 该群组的设置对象.
        """
        from . import get_running

        return await get_running().getGroupConfig(self)

    async def modifyConfig(self, config: "GroupConfig") -> None:
        """修改该群组的 Config

        Args:
            config (GroupConfig): 经过修改后的群设置对象.
        """
        from . import get_running

        return await get_running().modifyGroupConfig(self, config)

    async def getAvatar(self, cover: Optional[int] = None) -> bytes:
        """获取该群组的头像
        Args:
            cover (Optional[int]): 群封面标号 (若为 None 则获取该群头像, 否则获取该群封面)

        Returns:
            bytes: 群头像的二进制内容.
        """
        from . import get_running

        cover = (cover or 0) + 1
        session = get_running().adapter.session
        if not session:
            raise RuntimeError("No running ClientSession")
        return await (await session.get(f"https://p.qlogo.cn/gh/{self.id}/{self.id}_{cover}/")).content.read()


class Member(AriadneBaseModel):
    """描述用户在群组中所具备的有关状态, 包括所在群组, 群中昵称, 所具备的权限, 唯一ID."""

    id: int
    """QQ 号"""

    name: str = Field(..., alias="memberName")
    """显示名称"""

    permission: MemberPerm
    """群权限"""

    specialTitle: Optional[str] = None
    """特殊头衔"""

    joinTimestamp: Optional[int] = None
    """加入的时间"""

    lastSpeakTimestamp: Optional[int] = None
    """最后发言时间"""

    mutetimeRemaining: Optional[int] = None
    """禁言剩余时间"""

    group: Group
    """所在群组"""

    def __str__(self) -> str:
        return f"{self.name}({self.id} @ {self.group})"

    def __int__(self):
        return self.id

    async def getProfile(self) -> "Profile":
        """获取该群成员的 Profile

        Returns:
            Profile: 该群成员的 Profile 对象
        """
        from . import get_running

        return await get_running().getMemberProfile(self)

    async def getInfo(self) -> "MemberInfo":
        """获取该成员的可修改状态

        Returns:
            MemberInfo: 群组成员的可修改状态
        """
        return MemberInfo(name=self.name, specialTitle=self.specialTitle)

    async def modifyInfo(self, info: "MemberInfo") -> None:
        """
        修改群组成员的可修改状态; 需要具有相应权限(管理员/群主).

        Args:
            info (MemberInfo): 已修改的指定群组成员的可修改状态

        Returns:
            None: 没有返回.
        """
        from . import get_running

        return await get_running().modifyMemberInfo(self, info)

    async def modifyAdmin(self, assign: bool) -> None:
        """
        修改一位群组成员管理员权限; 需要有相应权限(群主)

        Args:
            assign (bool): 是否设置群成员为管理员.

        Returns:
            None: 没有返回.
        """
        from . import get_running

        return await get_running().modifyMemberAdmin(assign, self)

    async def getAvatar(self, size: Literal[640, 140] = 640) -> bytes:
        """获取该群成员的头像

        Args:
            size (Literal[640, 140]): 头像尺寸

        Returns:
            bytes: 群成员头像的二进制内容.
        """
        from . import get_running

        session = get_running().adapter.session
        if not session:
            raise RuntimeError("No running ClientSession")
        return await (await session.get(f"https://q.qlogo.cn/g?b=qq&nk={self.id}&s={size}")).content.read()


class Friend(AriadneBaseModel):
    """描述 Tencent QQ 中的好友."""

    id: int
    """QQ 号"""

    nickname: str
    """昵称"""

    remark: str
    """自行设置的代称"""

    def __int__(self):
        return self.id

    def __str__(self) -> str:
        return f"{self.remark}({self.id})"

    async def getProfile(self) -> "Profile":
        """获取该好友的 Profile

        Returns:
            Profile: 该好友的 Profile 对象
        """
        from . import get_running

        return await get_running().getFriendProfile(self)

    async def getAvatar(self, size: Literal[640, 140] = 640) -> bytes:
        """获取该好友的头像

        Args:
            size (Literal[640, 140]): 头像尺寸

        Returns:
            bytes: 好友头像的二进制内容.
        """
        from . import get_running

        session = get_running().adapter.session
        if not session:
            raise RuntimeError("No running ClientSession")
        return await (await session.get(f"https://q.qlogo.cn/g?b=qq&nk={self.id}&s={size}")).content.read()


class Stranger(AriadneBaseModel):
    """描述 Tencent QQ 中的陌生人."""

    id: int
    """QQ 号"""

    nickname: str
    """昵称"""

    remark: str
    """自行设置的代称"""

    def __int__(self):
        return self.id

    def __str__(self) -> str:
        return f"Stranger({self.id}, {self.nickname})"

    async def getAvatar(self, size: Literal[640, 140] = 640) -> bytes:
        """获取该陌生人的头像

        Args:
            size (Literal[640, 140]): 头像尺寸

        Returns:
            bytes: 陌生人头像的二进制内容.
        """
        from . import get_running

        session = get_running().adapter.session
        if not session:
            raise RuntimeError("No running ClientSession")
        return await (await session.get(f"https://q.qlogo.cn/g?b=qq&nk={self.id}&s={size}")).content.read()


class GroupConfig(AriadneBaseModel):
    """描述群组各项功能的设置."""

    name: str = ""
    """群名"""

    announcement: str = ""
    """群公告"""

    confessTalk: bool = False
    """开启坦白说"""

    allowMemberInvite: bool = False
    """允许群成员直接邀请入群"""

    autoApprove: bool = False
    """自动通过加群申请"""

    anonymousChat: bool = False
    """允许匿名聊天"""


class MemberInfo(AriadneBaseModel):
    """描述群组成员的可修改状态, 修改需要管理员/群主权限."""

    name: str = ""
    """昵称, 与 nickname不同"""

    specialTitle: Optional[str] = ""
    """特殊头衔"""


class DownloadInfo(AriadneBaseModel):
    """描述一个文件的下载信息."""

    sha: str = ""
    """文件 SHA256"""

    md5: str = ""
    """文件 MD5"""

    download_times: int = Field(..., alias="downloadTimes")
    """下载次数"""

    uploader_id: int = Field(..., alias="uploaderId")
    """上传者 QQ 号"""

    upload_time: datetime = Field(..., alias="uploadTime")
    """上传时间"""

    last_modify_time: datetime = Field(..., alias="lastModifyTime")
    """最后修改时间"""

    url: Optional[str] = None
    """下载 url"""


class Announcement(AriadneBaseModel):
    """群公告"""

    group: Group
    """公告所在的群"""

    senderId: int
    """发送者QQ号"""

    fid: str
    """公告唯一标识ID"""

    allConfirmed: bool
    """群成员是否已全部确认"""

    confirmedMembersCount: int
    """已确认群成员人数"""

    publicationTime: datetime
    """公告发布时间"""


class FileInfo(AriadneBaseModel):
    """群组文件详细信息"""

    name: str = ""
    """文件名"""

    path: str = ""
    """文件路径的字符串表示"""

    id: Optional[str] = ""
    """文件 ID"""

    parent: Optional["FileInfo"] = None
    """父文件夹的 FileInfo 对象, 没有则表示存在于根目录"""

    contact: Optional[Union[Group, Friend]] = None
    """文件所在位置 (群组)"""

    is_file: bool = Field(..., alias="isFile")
    """是否为文件"""

    is_directory: bool = Field(..., alias="isDirectory")
    """是否为目录"""

    download_info: Optional[DownloadInfo] = Field(None, alias="downloadInfo")
    """下载信息"""

    @validator("contact", pre=True, allow_reuse=True)
    def _(cls, val: Optional[dict]):
        if not val:
            return None
        if "remark" in val:  # Friend
            return Friend.parse_obj(val)
        return Group.parse_obj(val)  # Group


FileInfo.update_forward_refs(FileInfo=FileInfo)


class UploadMethod(str, Enum):
    """用于向 `upload` 系列方法描述上传类型"""

    Friend = "friend"
    """好友"""

    Group = "group"
    """群组"""

    Temp = "temp"
    """临时消息"""

    def __str__(self) -> str:
        return self.value


class CallMethod(str, Enum):
    """
    用于向 `Adapter.call_api` 指示操作类型.
    """

    GET = "GET"
    POST = "POST"
    RESTGET = "get"
    RESTPOST = "update"
    MULTIPART = "multipart"


class Client(AriadneBaseModel):
    """
    指示其他客户端
    """

    id: int
    """客户端 ID"""

    platform: str
    """平台字符串表示"""


class Profile(AriadneBaseModel):
    """
    指示某个用户的个人资料
    """

    nickname: str
    """昵称"""

    email: Optional[str]
    """电子邮件地址"""

    age: Optional[int]
    """年龄"""

    level: int
    """QQ 等级"""

    sign: str
    """个性签名"""

    sex: Literal["UNKNOWN", "MALE", "FEMALE"]
    """性别"""


class BotMessage(AriadneBaseModel):
    """
    指示 Bot 发出的消息.
    """

    messageId: int
    """消息 ID"""

    origin: Optional["MessageChain"]
    """原始消息链 (发送的消息链)"""


class AriadneStatus(Enum):
    """指示 Ariadne 状态的枚举类"""

    STOP = "stop"
    """已停止"""

    LAUNCH = "launch"
    """正在启动"""

    RUNNING = "running"
    """正常运行"""

    SHUTDOWN = "shutdown"
    """刚开始关闭"""

    CLEANUP = "cleanup"
    """清理残留任务"""
